package edu.travel.filter;

/**
 * TokenAuthenticationFilter 类。
 * <p>
 * 描述：
 *
 * @author huangwenwen
 * @date 2025/3/13
 */

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import edu.travel.entity.AccountInfo;
import edu.travel.service.BtAccountInfoService;
import edu.travel.util.EncryptUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

@Component
public class TokenAuthenticationFilter extends OncePerRequestFilter {

    @Autowired
    private BtAccountInfoService btAccountInfoService;  // 在单体应用中用数据库查询

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
            throws ServletException, IOException {

        String uri = request.getRequestURI();
        if (uri.startsWith("/baby/login")) {
            filterChain.doFilter(request, response);
            return;
        }
        // 从请求头获取 token
        String token = request.getHeader("Authorization");

        if (StringUtils.isNotBlank(token) && token.startsWith("Bearer ")) {
            try {
                String realToken = token.replace("Bearer ", "");
                String json = EncryptUtil.decodeUTF8StringBase64(realToken);
                JSONObject jsonObject = JSON.parseObject(json);

                // 获取用户名(电话)
                String phone = jsonObject.getString("principal");

                // 在单体应用中直接从数据库查询用户
                AccountInfo accountInfo = btAccountInfoService.findByPhone(phone);
                if (accountInfo == null) {
                    response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "User not found");
                    return;
                }

                // 获取权限
                String[] authorities = jsonObject.getJSONArray("authorities").toArray(new String[0]);

                // 创建 Spring Security 认证对象
                UsernamePasswordAuthenticationToken authenticationToken =
                        new UsernamePasswordAuthenticationToken(accountInfo, null, AuthorityUtils.createAuthorityList(authorities));
                authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));

                // 将认证对象放入 SecurityContext
                SecurityContextHolder.getContext().setAuthentication(authenticationToken);
            } catch (Exception e) {
                response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Invalid token");
                return;
            }
        }

        filterChain.doFilter(request, response);
    }
}
